//------------------------------------------------------------
/// \file	Main.cpp
/// \author	Rob Bateman
/// \date	9-feb-2005
/// \brief	This demonstrates a fairly easy way to wrap glut
/// 		keyboard input into something more useful for a
/// 		game engine. 
//------------------------------------------------------------

#include <stdlib.h>
#include <GL/glut.h>
#include "keys.h"

/// the position of the object
float g_position[2]={0,0};

//------------------------------------------------------------	Reshape()
//
void OnReshape(int w, int h)
{
	// prevent a division by zero
	if (h==0)
		h=1;

	// set the drawable region of the window
	glViewport(0,0,w,h);

	// set up the projection matrix 
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// just use an orthographic projection
	glOrtho(-50,50,-50,50,-100,100);

	// go back to modelview matrix so we can move the objects about
	glMatrixMode(GL_MODELVIEW);
}

//------------------------------------------------------------	Draw()
//
void OnDraw() {

	// clear the screen & depth buffer
	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	// clear the previous transform
	glLoadIdentity();

	// set position of object
	glTranslatef(g_position[0],g_position[1],0);

	// draw object
	glutWireSphere(1,50,50);

	// currently we've been drawing to the back buffer, we need
	// to swap the back buffer with the front one to make the image visible
	glutSwapBuffers();
}

//------------------------------------------------------------	OnInit()
//
void OnIdle() {

	// do all game input processing within the idle by polling the
	// state of the keyboard.
	// 
	if(IsKeyPressed(KEY_LEFT)) {
		g_position[0]-=0.01f;
	}
	if(IsKeyPressed(KEY_RIGHT)) {
		g_position[0]+=0.01f;
	}
	if(IsKeyPressed(KEY_UP)) {
		g_position[1]+=0.01f;
	}
	if(IsKeyPressed(KEY_DOWN)) {
		g_position[1]-=0.01f;
	}

	// redraw the screen
	glutPostRedisplay();
}


//------------------------------------------------------------	OnInit()
//
void OnInit() {
	// enable depth testing
	glEnable(GL_DEPTH_TEST);
}

//------------------------------------------------------------	OnExit()
//
void OnExit() {
}

//------------------------------------------------------------	OnKeyPress()
/// we can also create a callback to handle both special and 
/// normal key presses. This just checks for escape then quits
///
void OnKeyPress(int key,int x,int y) {
	if (key == KEY_ESCAPE)
		exit(EXIT_SUCCESS);
}

//------------------------------------------------------------	main()
//
int main(int argc,char** argv) {

	// initialise glut
	glutInit(&argc,argv);

	// request a depth buffer, RGBA display mode, and we want double buffering
	glutInitDisplayMode(GLUT_DEPTH|GLUT_RGBA|GLUT_DOUBLE);

	// set the initial window size
	glutInitWindowSize(640,480);

	// create the window
	glutCreateWindow("A basic glut example");

	// set the function to use to draw our scene
	glutDisplayFunc(OnDraw);

	// set the function to handle changes in screen size
	glutReshapeFunc(OnReshape);

	// ste the function to handle the updates for our program
	glutIdleFunc(OnIdle);
	
	// run our custom initialisation
	OnInit();

	// set the function to be called when we exit
	atexit(OnExit);

	// initialise the glut keyboard callbacks.
	InitKeyboard();

	// set a callback function for key presses
	SetKeyboardFunc(OnKeyPress);

	// this function runs a while loop to keep the program running.
	glutMainLoop();
	return 0;
}
